@@ -20,6 +20,8 @@
  * @APPLE_LICENSE_HEADER_END@
  */
 
+#undef DEBUG_CALLS
+
 #include <IOKit/audio/IOAudioControlUserClient.h>
 #include <IOKit/audio/IOAudioControl.h>
 #include <IOKit/audio/IOAudioTypes.h>
@@ -31,6 +33,22 @@
 #define super IOUserClient
 
 OSDefineMetaClassAndStructors(IOAudioControlUserClient, IOUserClient)
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 0);
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 1);
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 2);
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 3);
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 4);
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 5);
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 6);
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 7);
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 8);
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 9);
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 10);
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 11);
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 12);
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 13);
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 14);
+OSMetaClassDefineReservedUnused(IOAudioControlUserClient, 15);
 
 IOAudioControlUserClient *IOAudioControlUserClient::withAudioControl(IOAudioControl *control, task_t clientTask, void *securityID, UInt32 type)
 {
@@ -62,27 +80,17 @@ bool IOAudioControlUserClient::initWithAudioControl(IOAudioControl *control, tas
     clientTask = task;
     notificationMessage = 0;
 
-    methods[kAudioControlSetValue].object = this;
-    methods[kAudioControlSetValue].func = (IOMethod) &IOAudioControlUserClient::setControlValue;
-    methods[kAudioControlSetValue].count0 = 1;
-    methods[kAudioControlSetValue].count1 = 0;
-    methods[kAudioControlSetValue].flags = kIOUCScalarIScalarO;
-
-    methods[kAudioControlGetValue].object = this;
-    methods[kAudioControlGetValue].func = (IOMethod) &IOAudioControlUserClient::getControlValue;
-    methods[kAudioControlGetValue].count0 = 0;
-    methods[kAudioControlGetValue].count1 = 1;
-    methods[kAudioControlGetValue].flags = kIOUCScalarIScalarO;
-
     return true;
 }
 
 void IOAudioControlUserClient::free()
 {
-    clientClose();
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioControlUserClient[%p]::free()\n", this);
+#endif
     
     if (notificationMessage) {
-        IOFree(notificationMessage, sizeof(IOAudioNotificationMessage));
+        IOFreeAligned(notificationMessage, sizeof(IOAudioNotificationMessage));
         notificationMessage = 0;
     }
 
@@ -91,7 +99,11 @@ void IOAudioControlUserClient::free()
 
 IOReturn IOAudioControlUserClient::clientClose()
 {
-    if (audioControl) {
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioControlUserClient[%p]::clientClose()\n", this);
+#endif
+
+    if (audioControl && !isInactive()) {
         audioControl->clientClosed(this);
         audioControl = 0;
     }
@@ -101,6 +113,10 @@ IOReturn IOAudioControlUserClient::clientClose()
 
 IOReturn IOAudioControlUserClient::clientDied()
 {
+#ifdef DEBUG_CALLS
+    IOLog("IOAudioControlUserClient[%p]::clientDied()\n", this);
+#endif
+
     return clientClose();
 }
 
@@ -110,23 +126,27 @@ IOReturn IOAudioControlUserClient::registerNotificationPort(mach_port_t port,
 {
     IOReturn result = kIOReturnSuccess;
 
-    if (notificationMessage == 0) {
-        notificationMessage = (IOAudioNotificationMessage *)IOMalloc(sizeof(IOAudioNotificationMessage));
-        if (!notificationMessage) {
-            return kIOReturnNoMemory;
+    if (!isInactive()) {
+        if (notificationMessage == 0) {
+            notificationMessage = (IOAudioNotificationMessage *)IOMallocAligned(sizeof(IOAudioNotificationMessage), sizeof (IOAudioNotificationMessage *));
+            if (!notificationMessage) {
+                return kIOReturnNoMemory;
+            }
         }
+    
+        notificationMessage->messageHeader.msgh_bits = MACH_MSGH_BITS(MACH_MSG_TYPE_COPY_SEND, 0);
+        notificationMessage->messageHeader.msgh_size = sizeof(IOAudioNotificationMessage);
+        notificationMessage->messageHeader.msgh_remote_port = port;
+        notificationMessage->messageHeader.msgh_local_port = MACH_PORT_NULL;
+        notificationMessage->messageHeader.msgh_reserved = 0;
+        notificationMessage->messageHeader.msgh_id = 0;
+    
+        notificationMessage->type = type;
+        notificationMessage->ref = refCon;
+    } else {
+        result = kIOReturnNoDevice;
     }
-
-    notificationMessage->messageHeader.msgh_bits = MACH_MSGH_BITS(MACH_MSG_TYPE_COPY_SEND, 0);
-    notificationMessage->messageHeader.msgh_size = sizeof(IOAudioNotificationMessage);
-    notificationMessage->messageHeader.msgh_remote_port = port;
-    notificationMessage->messageHeader.msgh_local_port = MACH_PORT_NULL;
-    notificationMessage->messageHeader.msgh_reserved = 0;
-    notificationMessage->messageHeader.msgh_id = 0;
-
-    notificationMessage->type = type;
-    notificationMessage->ref = refCon;
-
+    
     return result;
 }
 
@@ -140,82 +160,3 @@ void IOAudioControlUserClient::sendValueChangeNotification()
         }
     }
 }
-
-IOExternalMethod *IOAudioControlUserClient::getExternalMethodForIndex(UInt32 index)
-{
-    IOExternalMethod *method = 0;
-
-    if (index < IOAUDIOCONTROL_NUM_CALLS) {
-        method = &methods[index];
-    }
-
-    return method;
-}
-
-IOReturn IOAudioControlUserClient::setControlValue(UInt32 value)
-{
-    IOReturn result = kIOReturnError;
-    
-    if (audioControl) {
-        IOCommandGate *cg;
-        
-        cg = audioControl->getCommandGate();
-        
-        if (cg) {
-            cg->runAction(IOAudioControl::setValueAction, (void *)value);
-        }
-    }
-    
-    return result;
-}
-
-IOReturn IOAudioControlUserClient::getControlValue(UInt32 *value)
-{
-    if (audioControl) {
-        *value = audioControl->getValue();
-    } else {
-        return kIOReturnError;
-    }
-
-    return kIOReturnSuccess;
-}
-
-IOReturn IOAudioControlUserClient::setProperties(OSObject *properties)
-{
-    OSDictionary *props;
-    IOReturn result = kIOReturnError;
-
-    if (audioControl && properties && (props = OSDynamicCast(OSDictionary, properties))) {
-        OSCollectionIterator *iterator;
-        OSObject *iteratorKey;
-
-        iterator = OSCollectionIterator::withCollection(props);
-        if (iterator) {
-            while (iteratorKey = iterator->getNextObject()) {
-                OSSymbol *key;
-
-                key = OSDynamicCast(OSSymbol, iteratorKey);
-                if (key && key->isEqualTo(IOAUDIOCONTROL_VALUE_KEY)) {
-                    OSNumber *value = OSDynamicCast(OSNumber, props->getObject(key));
-                    if (value) {
-                        IOCommandGate *cg;
-                        
-                        assert(audioControl);
-                        
-                        cg = audioControl->getCommandGate();
-                        
-                        if (cg) {
-                            result = cg->runAction(IOAudioControl::setValueAction, (void *)value->unsigned32BitValue());
-                        }
-                    }
-                }
-            }
-
-            iterator->release();
-        }
-    } else {
-        result = kIOReturnBadArgument;
-    }
-
-    return result;
-}
